#!/usr/bin/perl -w
# -*-Perl-*-
# Last changed Time-stamp: <2006-02-08 12:22:00 ivo>

require 5.004;

use Tk::Canvas;
use Tk::Scrollbar;
use strict;

my $rcsid = '$Id: AliDot.pl,v 1.7 2006/05/10 15:28:29 ivo Exp $'; #'

$, = ' ';
#some global variables
my $size = 600;    # initial size of dot plot canvas
my $ssize = $size; # size of scrollregion
my $magstep = 0;   # zoomfactor
my $length;        # sequence length
my $scalef;        # scaling factor (depends on size, magstep and length)
my $minread = sqrt(0.02);  # ignore entries with p<0.02
my $o_scalef;      # scaling factor at last redraw

my $main = new MainWindow;
# main window
die "usage: $0 alidot.out\n" if ($#ARGV != 0);
my $infile = $ARGV[0];

$main->title("Ivo's Tk Dot Plot Viewer");

# frame for buttons and controls
my $cframe = $main->Frame;
$cframe->pack(-fill => 'x');
$cframe->Button(-text => 'Quit',
		-command => sub{Tk::exit}
		)->pack(-side => 'right');
$cframe->Button(-text => 'Redraw',
		-command => \&draw_boxes
		)->pack(-side => 'right');

$cframe->Button(-text => 'List',
		-command => sub {
		    text_window($infile, $main)
		    }
		)->pack(-side => 'right');

my $ps = $cframe->Button(-text => 'Save Screen',
			 -command => \&make_ps
			 )->pack(-side => 'right');

my $HELP;
$cframe->Button(-text => 'Help', -command => \&show_help
		)->pack(-side => 'right');

# field to display info on current pair
my $CurrentPair="";
my $Info_field = $cframe->Message(qw/-width 12c -textvariable/
				=> \$CurrentPair);
$Info_field->pack(qw/-side left/);

# canvas for dot plot
my $canvas = $main->Scrolled(qw/Canvas -relief sunken -borderwidth 2
			     -scrollbars se
			     -width/ => $size, -height => $size,
			     -scrollregion => [0,0,$ssize, $ssize],
			     -background => 'white');
$canvas->pack(-expand => 'yes',  -fill => 'both', -padx => 1, -pady =>1);


#print times, "\n";
print STDERR "reading aliout...\n";
my (%pinfo, %ubox, %lbox, $struct, $seq);  # filled in by read_dot_plot
&read_aliout;
print STDERR "\n";
#print times, "\n";

$length = length($struct);
$scalef = $size/($length);

my $minp = 1/(4*$scalef); $minp = $minp/(1+$minp);

$canvas->createLine(0,0,$ssize,$ssize);
&draw_boxes;

$main->update();

my $margin = $canvas->Width - $size;  # $canvas->Width includes scrollbars

&add_bindings;
#print times, "\n";

Tk::MainLoop;

# end of main

sub draw_boxes {
    my $pp;
    my $orig_cursor = ($main->configure(-cursor))[3];
    $main->configure(-cursor => 'watch');
    $main->update();
    printf STDERR "drawing boxes... %d %5.3f %4.2f", $ssize, $minp*$minp, 1/(4*$scalef);
    $canvas->delete('all');

    # work around drawing problems when line is too long (does Tk use short??)
    # replaces a simple  $canvas->createLine(0,0,$ssize,$ssize);
    my ($x0, $y0) = (1, 1);
    while (($x0<$ssize) && ($y0<$ssize)) {
	my ($x1, $y1) = ($ssize, $ssize);
	$x1 = $x0 + 5000 if $x1 >  $x0 + 5000;
	$y1 = $y0 + 5000 if $y1 >  $y0 + 5000;
	$canvas->createLine($x0-1, $y0-1, $x1, $y1);
	($x0,$y0) = ($x1,$y1);
    }

    $canvas->CanvasFocus;  # focus  canvas so that '+' and '-' key work

    foreach $pp (keys %pinfo) {
	my ($i,$j) = split($;,$pp);
	my ($r, $nseq, $p, $s, $ncomp, @bp) = @{$pinfo{$pp}};
	if ($p>=$minp) {
	    my $hue = ($ncomp-1.0)/6.2;
	    my $sat = 1 - $nseq/2.5;
	    $sat = 0 if ($sat<0);
	    my $c = hsvToRgb($hue, $sat, 1-0.3*$sat);
	    make_box($j, $i, $p, $c, $bp[$#bp]);
	}
    }

    $main->configure(-cursor => $orig_cursor);
    $o_scalef = $scalef;
    print STDERR "\n";
}

sub make_box {
    my ($i, $j, $p, $c, $plus) = @_;
    my $pp = $p*$scalef;

    my $x1 = ($i-0.5-$p/2)*$scalef;  # centered between i and i+1
    my $y1 = ($j-0.5-$p/2)*$scalef;

    $canvas->createRectangle($x1,  $y1, $x1+$pp, $y1+$pp,
			     -fill => $c, -outline => undef);
    $canvas->createRectangle($y1,  $x1, $y1+$pp, $x1+$pp,
			     -fill => $c, -outline => undef)
	unless ($plus eq '+');
}

sub add_bindings {
    $canvas->CanvasBind('<Motion>' => sub {
	my $c = shift;
	my $e = $c->XEvent;
	my $x = $canvas->canvasx($e->x);
	$x = int($x/$scalef +1);
	my $y = $canvas->canvasy($e->y);
	$y = int($y/$scalef +1);
	$CurrentPair = "$y,$x";
    });
    # button 2 scrolls
    my ($scanX, $scanY);
    $canvas->CanvasBind('<2>' => sub {
	my $c = shift;
	my $e = $c->XEvent;
	$scanX = $e->x; $scanY = $e->y;
	$c->scan('mark', $e->x, $e->y);
    });
    $canvas->CanvasBind('<B2-Motion>' => sub {
	my $c = shift;
	my $e = $c->XEvent;
	my $x = $scanX + ($e->x -$scanX)/10;
	my $y = $scanY + ($e->y -$scanY)/10;
	$c->scan('dragto', $x, $y);
#	$canvas->xview(scroll => $e->x - $scanX, 'unit');
#	$canvas->yview(scroll => $e->y - $scanY, 'unit');
    });

    $canvas->CanvasBind('<1>' => sub {
#	my $c = shift;
#	my $e = $c->XEvent;
#	print $e->x, ' ', $e->y, "\n";
	my ($i, $j) = split(/[, ]/,$CurrentPair);
	if ($i>$j) {
	    my $ii = $i;
	    $i =$j; $j=$ii;
	}
	$CurrentPair = "$i,$j ";
	if (exists $pinfo{$i,$j}) {
	    my ($r, $nseq, $p, $s, $ncomp, @bp) = @{$pinfo{$i,$j}};
	    $p *= $p;
	    $CurrentPair .= "rank=$r nc=$nseq p=$p @bp";
	}
    });

    # rescale everything if window is resized
    $canvas->CanvasBind('<Configure>' => sub {
	my $new_size = $canvas->Width;
	$new_size = $canvas->Height if ($canvas->Height<$new_size);
	$new_size -= $margin;
	if ($new_size>$ssize) {
	    my $rscale = $new_size/$ssize;
	    $canvas->scale('all', 0, 0, $rscale, $rscale);
	    $scalef *= $rscale;
	    $size  = $ssize = $new_size;
	    $canvas->configure(-scrollregion => [0,0,$ssize, $ssize]);
	} else {
	    $size = $new_size;
	}
    });


    # zoom in and out
    $main->bind('<plus>' => \&zoom_in);
    $main->bind('<KP_Add>' => \&zoom_in);
    $main->bind('<minus>' => \&zoom_out);
    $main->bind('<KP_Subtract>' => \&zoom_out);
    
    # KP_Home etc do not always exist. 
    # The eval makes sure we don't die from an undefined ksym
    eval { 
	$canvas->CanvasBind('<KP_Home>' => sub {
	    $canvas->xview(scroll => -1 , 'unit');
	    $canvas->yview(scroll => -1 , 'unit');
	});
	$canvas->CanvasBind('<KP_Next>' => sub {
	    $canvas->xview(scroll => 1 , 'unit');
	    $canvas->yview(scroll => 1 , 'unit');
	});
	$canvas->CanvasBind('<KP_Prior>' => sub {
	    $canvas->xview(scroll =>  1 , 'unit');
	    $canvas->yview(scroll => -1 , 'unit');
	});
	$canvas->CanvasBind('<KP_End>' => sub {
	    $canvas->xview(scroll => -1 , 'unit');
	    $canvas->yview(scroll =>  1 , 'unit');
	});
    }
}

sub zoom_in {
    $scalef *= 1.2;
    $ssize *= 1.2;
    my ($x1, $x2) = $canvas->xview;
    my ($y1, $y2) = $canvas->yview;
    $canvas->configure(-scrollregion => [0,0,$ssize, $ssize]);
    $canvas->xview(moveto => $x1+($x2-$x1)*0.1);
    $canvas->yview(moveto => $y1+($y2-$y1)*0.1);
    $canvas->scale('all', 0, 0, 1.2, 1.2);

    $minp = 1/(4*$scalef); $minp = $minp/(1+$minp);
    &draw_boxes if ($scalef>2*$o_scalef) && (10*$minp>$minread);
}

sub zoom_out {
    my $f = 1/1.2;
    $f = $size/$ssize if ($ssize*$f < $size); # keep $ssize>=$size
    $ssize *= $f;
    $scalef *= $f;
    my ($x1, $x2) = $canvas->xview;
    my ($y1, $y2) = $canvas->yview;
    $canvas->configure(-scrollregion => [0,0,$ssize, $ssize]);
    my $ff = ($f-1)/2;
    $canvas->xview(moveto => $x1+($x2-$x1)*$ff);
    $canvas->yview(moveto => $y1+($y2-$y1)*$ff);
    $canvas->scale('all', 0, 0, $f, $f);

    $minp = 1/(3*$scalef); $minp = $minp/(1+$minp);

    &draw_boxes if ($scalef<0.4*$o_scalef);;
}

sub jump_to {
    my($y, $x) = @_;
    my ($min, $max) = $canvas->xview;
    $max = $max - $min;
    $min = $x/$length - $max/2;
    $min = 0 if ($min < 0);
    $min = 1 - $max if ($min +$max>1);
    $canvas->xview(moveto => $min);
    ($min, $max) = $canvas->yview;
    $max = $max - $min;
    $min = $y/$length - $max/2;
    $min = 0 if ($min < 0);
    $min = 1 - $max if ($min +$max>1);
    $canvas->yview(moveto => $min);
}

sub read_aliout {
    $_ = <>; print;
    $_ = <>; print; # skip first two lines
    while (<>) {
	if (/\.\.\./) {
	    chomp;
	    $struct = $_;
	} else {
	    my ($i,$j,$nc,$p,$s,@bp) = split;
	    next unless($j>0);
	    chop $p; $p = sqrt($p/100);
	    my($ncomp, $pp) = 0;
	    foreach $pp (@bp) {
		$ncomp++ if ($pp =~ /[AUCG]{2}:\d/);
	    }
	    next if $p<$minread;
	    $pinfo{$i,$j} = [$.-2, $nc, $p, $s, $ncomp, @bp];
	}
    }
}

# modified from Tk::ColorSelect
sub hsvToRgb {

    # The procedure below converts an HSB value to RGB.  It takes hue,
    # saturation, and value components (floating-point, 0-1.0) as arguments,
    # and returns a list containing RGB components (integers, 0-65535) as
    # result.  The code here is a copy of the code on page 616 of
    # "Fundamentals of Interactive Computer Graphics" by Foley and Van Dam.

    my($hue, $sat, $value) = @_;
    my($v, $i, $f, $p, $q, $t);

    $v = (0 .. 9, 'a' .. 'f')[int(15 * $value)];
    return "#$v$v$v" if $sat == 0;
    $hue *= 6;
    $hue = 0 if $hue >= 6;
    $i = int($hue);
    $f = $hue - $i;
    $p = (0 .. 9, 'a' .. 'f')[int(15 * $value * (1 - $sat))];
    $q = (0 .. 9, 'a' .. 'f')[int(15 * $value * (1 - ($sat * $f)))];
    $t = (0 .. 9, 'a' .. 'f')[int(15 * $value * (1 - ($sat * (1 - $f))))];
    return "#$v$t$p" if $i == 0;
    return "#$q$v$p" if $i == 1;
    return "#$p$v$t" if $i == 2;
    return "#$p$q$v" if $i == 3;
    return "#$t$p$v" if $i == 4;
    return "#$v$p$q" if $i == 5;

} # end hsvToRgb

sub show_help {
    my $help_text = <<EOF;
Tk dot plot viewer $rcsid.

Hit '+' and '-' to zoom in or out.
    Use the middle mouse button drag the canvas, or the cursor keys to scroll.

The base pair currently under the mouse is indicated in the upper left corner.
Left-click on a square to display additional information about the pair:
    the 'rank' of the pair (sorted by 'credibility')
    the number 'nc' of sequences not compatible with the pair
    its mean frequency 'p'
    a list showing how often each type of pair occurs
a trailing '+' indicates the pair is incompatible with a higher ranked pair.

Click the 'List'-button to display the ranked list of base pairs in a separate
text window. Clicking on a line in the text window will center the canvas on
the corresponding base pair.

Click on 'Save Screen' to print or save the current view in a PostScript file.

EOF

    if (!defined $HELP) {
	$HELP = $main->Toplevel;
	$HELP->Button(-text    => 'Dismiss',
		      -command => [$HELP => 'withdraw'])->pack;
#	$VIEW->Scrolled('Text',
#			qw/-scrollbars e -height 40 -setgrid 1/)->pack;
	$HELP->Message(qw/-width 15c -text/ => $help_text)->pack;
    } else {
	$HELP->deiconify;
	$HELP->raise;
    }
}

sub text_window
{
    my ($filename, $whandle) = @_;
#    print"XXXX $filename, $whandle XXXX\n";

    # Create a toplevel textwindow
    my $texttop = $whandle->Toplevel();
    $texttop->wm('title', $filename);
    $texttop->Button(-text => 'Exit',
		     -command => sub {$texttop->destroy()}
		     )->pack();

    # Create a textwindow
    my $textwin = $texttop->Text()->pack(-side => 'left',
					   -fill => 'both'
					   );
    $textwin->bind('<1>' => sub {
	my $w = shift;
	my $pos = $w->XEvent->xy;
	$textwin->tag('add', 'sel', "$pos linestart", "$pos lineend");
#	my ($j, $j) = split(' ', $textwin->get("$pos linestart", "$pos lineend"), 3);
	my ($i, $j) = split(' ', $textwin->get('sel.first', 'sel.last'));
	&jump_to($i, $j);
    });

    # Create a scrollbar and inform it about the textwindow
    my $texscroll = $texttop->Scrollbar(-command => [yview => $textwin]
				     )->pack(-side => 'right',
					     -fill => 'y'
					     );
    # Inform textwin about scrollbar
    $textwin->configure(-yscrollcommand => ['set', $texscroll]);

    # Insert text into textwindow
    $textwin->configure(-state => 'normal');
    my $blah =  `head -1000 $filename`;
    $textwin->insert('end', $blah);
    $textwin->configure(-state => 'disabled');
}

sub make_ps {
    my $top = new MainWindow;
    my $fn = 'canvas.ps';
    $top->title("Print");
    my $f = $top->Frame(qw/-borderwidth 2/)->pack(qw/-side top/);
    $f->Label(-text => 'Name of PS file')
	->pack(qw/-side left/);
    $f->Entry(-textvariable => \$fn)->pack(qw/-side right -fill x/);
    $top->Button(-text => 'Save',
		 -command => sub{
		     $canvas->postscript(-file => $fn);
		     $top->destroy()
		     }
		 )->pack(-side => 'left');
    $top->Button(-text => 'Cancel',
		 -command => sub {$top->destroy()}
		 )->pack(-side => 'right');
}

# End of file
