/*
	      postscript output for secondary structures
		    and pair probability matrices
				   
		 c Ivo L Hofacker and Peter F Stadler
			  Vienna RNA package
*/
/*Last changed Time-stamp: <2003-10-02 10:23:03 ivo> */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "ali.h"

static char rcsid[] = "$Id: PS_dot.c,v 1.4 2003/10/02 08:46:17 ivo Exp $";

extern void nrerror(char *message);

#define  PRIVATE   static
#define  MAX(A,B)    (A)>(B)?(A):(B)
#define  MAXLENGTH 1500
#ifndef PI
#define  PI       3.141592654
#endif
#define  PIHALF       PI/2.

#define  BRANCH     30

#define MIN2(A, B)      ((A) < (B) ? (A) : (B))

/*--------------------------------------------------------------------------*/

#define PMIN 0.00001
int PS_dot_plot(char *string, pair_info *pi, pair_info *ppi, char *wastlfile)
{
   FILE *wastl;
   int i, length;
   char name[31], *c;

   length= strlen(string);
   wastl = fopen(wastlfile,"w");
   if (wastl==NULL) {
     fprintf(stderr, "can't open %s for dot plot\n", wastlfile);
     return 0; /* return 0 for failure */
   }
   strncpy(name, wastlfile, 30);
   if ((c=strrchr(name, '_'))!=0) *c='\0';
   fprintf(wastl,"%%!PS-Adobe-3.0 EPSF-3.0\n");
   fprintf(wastl,"%%%%Title: RNA DotPlot\n");
   fprintf(wastl,"%%%%Creator: %s\n", rcsid+5);
   /* fprintf(wastl,"%%%%CreationDate: %s", time_stamp()); */
   fprintf(wastl,"%%%%BoundingBox: 66 211 518 662\n");
   fprintf(wastl,"%%%%DocumentFonts: Helvetica\n");
   fprintf(wastl,"%%%%Pages: 1\n");
   fprintf(wastl,"%%%%EndComments\n\n");
   
   fprintf(wastl,"%%This file contains the square roots "
	   "of the base pair probabilities in the form\n");
   fprintf(wastl,"%% i  j  sqrt(p(i,j)) ubox\n");
   fprintf(wastl,"100 dict begin\n");  /* DSC says EPS should create a dict */
   fprintf(wastl,"\n/logscale false def\n\n");
   fprintf(wastl,"%%delete next line to get rid of title\n"
	   "270 665 moveto /Helvetica findfont 14 scalefont setfont "
	   "(%s) show\n\n", name);
   fprintf(wastl,"/lpmin {\n"
	   "   %g log  %% log(pmin) only probs>pmin will be shown\n"
	   "} bind def\n\n",PMIN);
 
   /* EPS should not contain lines >255 characters */
   fprintf(wastl,"/sequence { (\\\n");
   i=0;
   while (i<length) {
     fprintf(wastl, "%.255s\\\n", string+i);
     i+=255;
   }
   fprintf(wastl,") } def\n");
   fprintf(wastl,"/len { sequence length } def\n\n");  

   fprintf(wastl,"/ubox {\n"     /* upper triangle matrix */
   /* bright = 1 - 0.3*sat */
	   "   dup 0.3 mul 1 exch sub sethsbcolor\n"
	   "   3 1 roll\n"
	   "   exch len exch sub 1 add box\n"
	   "} bind def\n\n");
   
   fprintf(wastl,"/lbox {\n"     /* lower triangle matrix */
	   "   dup 0.3 mul 1 exch sub sethsbcolor\n"
	   "   3 1 roll\n"
	   "   len exch sub 1 add box\n"
	   "} bind def\n\n");

   fprintf(wastl,"/box { %%size x y box - draws box centered on x,y\n"
	   "   2 index 0.5 mul add            %% x += 0.5\n"
	   "   exch 2 index 0.5 mul add exch  %% x += 0.5\n"
	   "   newpath\n"
	   "   moveto\n"
	   "   dup neg   0 rlineto\n"
	   "   dup neg   0 exch rlineto\n"
	   "             0 rlineto\n"
	   "   closepath\n"
	   "   fill\n"
	   "} def\n\n");
   
   fprintf(wastl,"72 216 translate\n");
   fprintf(wastl,"72 6 mul len 1 add div dup scale\n");
   fprintf(wastl,"/Helvetica findfont 0.95 scalefont setfont\n\n");

   /* print sequence along all 4 sides */
   fprintf(wastl,"%% print sequence along all 4 sides\n");
   fprintf(wastl,"0 1 len 1 sub {\n");
   fprintf(wastl,"    dup\n");
   fprintf(wastl,"    0.7 add -0.3 moveto\n");
   fprintf(wastl,"    sequence exch 1 getinterval\n");
   fprintf(wastl,"    show\n");
   fprintf(wastl,"} for\n");
   fprintf(wastl,"\n");
   fprintf(wastl,"0 1 len 1 sub {\n");
   fprintf(wastl,"    dup\n");
   fprintf(wastl,"    0.7 add 0.7 len add moveto\n");
   fprintf(wastl,"    sequence exch 1 getinterval\n");
   fprintf(wastl,"    show\n");
   fprintf(wastl,"} for\n\n");

   fprintf(wastl,"90  rotate\n");
   fprintf(wastl,"0 1 len 1 sub {\n");
   fprintf(wastl,"    dup\n");
   fprintf(wastl,"    0.7 add -0.2 moveto\n");
   fprintf(wastl,"    len 1 sub exch sub\n");
   fprintf(wastl,"    sequence exch 1 getinterval\n");
   fprintf(wastl,"    show\n");
   fprintf(wastl,"} for\n");
   fprintf(wastl,"270 rotate\n\n");

   fprintf(wastl,"270 rotate\n");
   fprintf(wastl,"0 1 len 1 sub {\n");
   fprintf(wastl,"    dup\n");
   fprintf(wastl,"    -0.3 add len sub  0.7 len add  moveto\n");
   fprintf(wastl,"    sequence exch 1 getinterval\n");
   fprintf(wastl,"    show\n");
   fprintf(wastl,"} for\n");
   fprintf(wastl,"90 rotate\n\n");

   /* do grid */
   fprintf(wastl,"0.5 dup translate\n"
	   "%% draw diagonal\n"
	   "0.04 setlinewidth\n"
	   "0 len moveto len 0 lineto stroke \n\n");
   fprintf(wastl,"%%draw grid\n"
	   "0.01 setlinewidth\n"
	   "len log 0.9 sub cvi 10 exch exp  %% grid spacing\n"
	   "dup 1 gt {\n"
	   "   dup dup 20 div dup 2 array astore exch 40 div setdash\n"
	   "} { [0.3 0.7] 0.1 setdash } ifelse\n"
	   "0 exch len {\n"      /* for (i=0; i<=len; i++) */
	   "   dup dup\n"        
	   "   0 moveto\n"                     /* i 0 moveto   */
	   "   len lineto \n"                  /* i len lineto */
	   "   dup\n"
	   "   len exch sub 0 exch moveto\n"   /* 0 i moveto   */
	   "   len exch len exch sub lineto\n" /* len i lineto */
	   "   stroke\n"
	   "} for\n"
	   "0.5 neg dup translate\n\n");

   /* print boxes, remember to add 1 to pi[].i and pi[].j */

   i=0;
   while (pi[i].j>0) {
      int ncomp, j;
      float pp, hue, sat;
      pp = sqrt(pi[i].p);
      for (ncomp=0, j=1; j<=6; j++) if (pi[i].bp[j]) ncomp++;
      hue = (ncomp-1.0)/6.2;   /* hue<6/6.9 since hue=1 is same as hue=0 */
      sat = 1 - MIN2( 1.0, pi[i].bp[0]/2.5);
      fprintf(wastl,"%d %d %1.4f %1.2f %1.2f ubox\n",
	      pi[i].i+1, pi[i].j+1, pp, hue, sat);
      i++;
   }
   fprintf(wastl,"\n");

   i=0;
   while (ppi[i].j>0) {
     int ncomp, j;
     float pp, hue, sat;
     pp = sqrt(ppi[i].p);
     for (ncomp=0, j=1; j<=6; j++) if (ppi[i].bp[j]) ncomp++;
     hue = (ncomp-1.0)/6.2;   /* hue<6/6.9 since hue=1 is same as hue=0 */
     sat = 1 - MIN2( 1.0, ppi[i].bp[0]/2.5);
     if (ppi[i].i<ppi[i].j)
       fprintf(wastl,"%d %d %1.4f %1.2f %1.2f lbox\n",
	       ppi[i].i+1, ppi[i].j+1, pp, hue, sat);
     else {
       fprintf(stderr, "unsorted pair\n");
       fprintf(wastl,"%d %d %1.4f %1.2f %1.2f lbox\n",
	       ppi[i].j+1, ppi[i].i+1, pp, hue, sat);
     }
     i++;
   }

   fprintf(wastl,"showpage\n");
   fprintf(wastl,"end\n");
   fprintf(wastl,"%%%%EOF\n");
   fclose(wastl);
   return 1; /* success */
}

/*---------------------------------------------------------------------------*/

